#ifndef OBJ16_DB_API_H
#define OBJ16_DB_API_H

/* RDM 11.0.1.551 */

#include "Db.h"
#include "Cursor.h"
#include "obj16_db.h"

namespace RDM_CPP {

class Cursor_rec01;

/*! \brief The obj16_db Db class
 *
 * This class provides the implementation for the obj16_db database
*/
class Db_obj16_db : public Db {
public:
    /*! \brief Open the obj16_db database using the default name
     *
     * \param mode The open mode to use
     * \return A validated Db_obj16_db object
     */
    static Db_obj16_db Open (char *mode="s") throw (rdm_exception&);

    /*! \brief Open the obj16_db database using the default name
     *
     * \param mode The open mode to use
     * \return A validated Db_obj16_db object
     */
#if !defined(RDM_NO_UNICODE)
    static Db_obj16_db Open (wchar_t *mode=L"s") throw (rdm_exception&);
#endif /*RDM_NO_UNICODE*/

    /*! \brief Open the obj16_db database using the specified name
     *
     * \param name The database name to use for the open call
     * \param mode The open mode to use
     * \return A validated Db_obj16_db object
     */
    static Db_obj16_db Open (const char *name, const char *mode="s") throw (rdm_exception&);

    /*! \brief Open the obj16_db database using the specified name
     *
     * \param name The database name to use for the open call
     * \param mode The open mode to use
     * \return A validated Db_obj16_db object
     */
#if !defined(RDM_NO_UNICODE)
    static Db_obj16_db Open (const wchar_t *name, const wchar_t *mode=L"s") throw (rdm_exception&);
#endif /*RDM_NO_UNICODE*/

    //! Db_obj16_db Default Constructor
    Db_obj16_db (void) throw ();
    /*! \brief Db_obj16_db Constructor
     *
     * \param db A Db object used to create the Db_obj16_db
     */
    Db_obj16_db (const Db& db) throw ();
    //! Db_obj16_db Copy Constructor
    Db_obj16_db (const Db_obj16_db&) throw ();

    //! Db_obj16_db assignment operator
    Db_obj16_db& operator = (const Db&) throw (rdm_exception&);

    //! Db_obj16_db assignment operator
    Db_obj16_db& operator = (const Db_obj16_db&) throw ();

    /*! \brief Db_obj16_db destructor
     *
     * Remove a reference to the underlying database.  The database is closed when the last reference is gone.
     */
    ~Db_obj16_db ();

    /*! \brief Create a new rec01 record
     *
     * \returns A singleton Cursor_rec01 object with default field values
     *
     * Create a new rec01 record using default values
     */
    Cursor_rec01 New_rec01_record (void) const throw (rdm_exception&);

    /*! \brief Create a new rec01 record
     *
     * \param fields The field values for the new rec01 record
     * \returns A singleton Cursor_rec01 object with the specified field values
     *
     * Create a new rec01 record using the specified values
     */
    Cursor_rec01 New_rec01_recordWithFieldValues (struct rec01 *fields) const throw (rdm_exception&);

    /*! \brief Obtain rec01 cursor position at the first rec01 record
     * \returns rec01 cursor positioned at the first rec01 record
     *
     * Create a new rec01 cursor for doing a table scan of the rec01 records.
     * The cursor is positioned at the first rec01 record
     */
    Cursor_rec01 Get_rec01_records (void) const throw (rdm_exception&);

    /*! \brief Obtain a rec01 cursor based on the field01 key
     * \returns rec01 cursor positioned at the first rec01 field01 key value
     *
     * Create a new rec01 cursor based on the field01 key and position the cursor
     * at the first record
     */
    Cursor_rec01 Get_rec01_recordsBy_field01 () const throw (rdm_exception&);

private:
    /*! \brief Db_obj16_db reference operator */
    Db_obj16_db* operator & (void);
    /*! \brief Db_obj16_db constructor */
    Db_obj16_db (IDb *idb);
    /*! \brief Get the Db_obj16_db dbType
     * \return The Db_obj16_db dbType
     */
    static dbType Type (void);
};

/*! \brief The rec01 Cursor class
 *
 * This class provides the implementation for the rec01 Cursor
*/
class Cursor_rec01 : public Cursor {
public:
    /*! \brief Get the type for objects of this class
     * \return The type for Cursor_rec01 objects
     */
    static recordType Type (void);

    /*! Cursor_rec01 Default Constructor */
    Cursor_rec01 (void) throw ();
    /*! \brief Cursor_rec01 Constructor
     *
     * \param cursor A Cursor object used to create the Cursor_rec01 object
     */
    Cursor_rec01 (const Cursor& cursor) throw (rdm_exception&);
    /*! \brief Cursor_rec01 Copy Constructor */
    Cursor_rec01 (const Cursor_rec01&) throw ();

    /*! \brief Cursor_rec01 assignment operator */
    Cursor_rec01& operator = (const Cursor& cursor) throw (rdm_exception&);

    /*! \brief Cursor_rec01 assignment operator */
    Cursor_rec01& operator = (const Cursor_rec01& cursor) throw ();

    /*! \brief Create a new rec01 cursor doing field01 key navigation.
     *
     * \return rec01 cursor ordered by the field01 key
     *
     *  Create a new rec01 cursor using field01 key navigation and position the
     *  new cursor at our current position
     */
    Cursor_rec01 GetRecordsBy_field01 () const throw (rdm_exception&);

    /*! \brief Position the rec01 cursor to the field01 key value
     *
     * \param field01 The field01 value to position the cursor at
     *
     */
    void KeyFindBy_field01 (int32_t *field01_val) const throw (rdm_exception&);

    /*! \brief Set all of the field values of the record at the current cursor position
     * \param fields A const pointer with the field values to be written to the current cursor position
     */
    void SetFieldValues (const struct rec01 *fields) const throw (rdm_exception&);

    /*! \brief Get all of the field values for the record at the current cursor position
     * \param fields A pointer where the record values from the current cursor position will be written
     */
    void GetFieldValues (struct rec01 *fields) const throw (rdm_exception&);

    /*! \brief Set the field01 field value for the current record
     * \param field01 The field value that will be written to the record at the current cursor position
     */
    void Set_field01 (int32_t field01_val) const throw (rdm_exception&);

    /*! \brief Get the field01 field value for the current record
     * \param field01 A pointer where the field value from the current cursor position will be written
     */
    void Get_field01 (int32_t *field01_val) const throw (rdm_exception&);

    /*! \brief Cursor_rec01 Destructor
     *
     * Release the cursor
     */
    ~Cursor_rec01 ();

private:
    friend class Db_obj16_db;
};

}
#endif    /* OBJ16_DB_API_H */
